/*
 
 //This file is licensed under the terms of the CC-LGPL
 
 CL_rlp_equation_test.pov
 
 Linking together two objects with a string between them. 
 The position of the centre of the rendered string is returned so you can line up
 several equations without trial and error.   
 
 Also indicates that even relatively simple mathematical expessions require construction in several stages.
 
*/
  ////
 //   example   //
//_____________//  
#global_settings{charset utf8 assumed_gamma 1.0}
//
#include "colors.inc"
#include "chars.inc"
#include "rlp.inc"
#include "CL_rlp_capital_letters_160714.inc" 
// 
CL_rlp_make_text_string_set(3) // Defines CL_tiny_space,CL_small_space,CL_medium_space,CL_large_space,CL_huge_space,CL_letter_adjust,
//                                        CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type as globals
// Make the derivative operator
#declare deriv_object         = object{CL_rlp_make_text_fraction(CL_tiny_space,CL_letter_adjust,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                  "|d","|d#x",CL_medium_space,CL_dimensions_out)}
#declare deriv_dimensions     = CL_dimensions_out; 
//
// Lower equation
//
// Make ln f(x)
#declare log_object           = object{CL_rlp_make_text_string("ln #f(x)",CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                CL_dimensions_out)} 
#declare log_dimensions       = CL_dimensions_out; 
// Put brackets around it
#declare delimited_object     = object{CL_rlp_make_elastic_bracket_pair(CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,
                                        CL_cham,CL_font_type,log_object,CL_tiny_space,1,1,log_dimensions,CL_dimensions_out)} 
#declare delimited_dimensions = CL_dimensions_out; 
// Concatenate the operator and the bracketed object
#declare part_dimensions      = array[3];   // Mostly only needs [2], but there is a make_equation using a middle_part object.
#declare part_dimensions[0]   = deriv_dimensions;
#declare part_dimensions[1]   = delimited_dimensions;
#declare concat_object        = object{CL_rlp_make_concatenate(deriv_object,delimited_object,CL_medium_space,part_dimensions,CL_dimensions_out)} 
//
#declare part_dimensions[0]   = CL_dimensions_out;
// Make the fraction
#declare fraction_object      = object{CL_rlp_make_text_fraction(CL_tiny_space,CL_letter_adjust,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                  "#f'(x)","#f(x)",CL_medium_space,CL_dimensions_out)}
#declare fraction_dimensions  = CL_dimensions_out;
#declare part_dimensions[1]   = CL_dimensions_out; 
#declare equation_object      = object{CL_rlp_make_equation(CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                       concat_object,fraction_object,0," = ",CL_medium_space,part_dimensions,CL_dimensions_out)} 
//
// Bottom equation
// The make equation #macro produces an object, but instead of using CL_dimensions_out[2] to mean the superscript height, it denotes the
// x co-ordinate of the centre of the equals sign (or other relational string) 
//
object{equation_object translate <-CL_dimensions_out[2],-5,0> pigment{rgb 1} finish{ambient 0 specular 1}}      
//
// Upper equation
//
#declare fun_object           = object{CL_rlp_make_text_string("#f(x)",CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                CL_dimensions_out)} 
#declare fun_dimensions       = CL_dimensions_out; 
//
#declare surd_object          = object{CL_rlp_make_elastic_surd(CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                            fun_object,CL_medium_space,0,fun_dimensions,CL_dimensions_out) }
#declare surd_dimensions      = CL_dimensions_out; 
//
#declare fundash_object       = object{CL_rlp_make_text_string("#f'(x)",CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                CL_dimensions_out)} 
#declare fundash_dimensions   = CL_dimensions_out; 
// 
#declare part_dimensions[0]   = fundash_dimensions;
#declare part_dimensions[1]   = surd_dimensions;
#declare fraction_object      = object{CL_rlp_make_fraction(CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,
                                                            fundash_object,surd_object,CL_medium_space,part_dimensions,CL_dimensions_out)}
#declare fraction_dimensions  = CL_dimensions_out;
//
#declare half_object          = object{CL_rlp_make_text_fraction(CL_tiny_space,CL_letter_adjust,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                  "#1","#2",CL_medium_space,CL_dimensions_out)}
#declare half_dimensions      = CL_dimensions_out;
//  If you fancy a smaller half object, you can scale, but you have to translate up to keep the object centred on y = 0.9.
//  And you have to change the x width. In another instance, you might need to scale the height above and drop below, so I do here.
#declare half_scale_fact      = 0.75;
#declare half_object          = object{half_object scale <half_scale_fact,half_scale_fact,1> translate <0,0.9*(1-half_scale_fact),0>}
#declare half_dimensions[0]   = half_dimensions[0]*half_scale_fact;
#declare half_dimensions[1]   = half_dimensions[1]*half_scale_fact + 0.9*(1-half_scale_fact);  
#declare half_dimensions[2]   = half_dimensions[1];
#declare half_dimensions[3]   = half_dimensions[3]*half_scale_fact - 0.9*(1-half_scale_fact); 
#if(half_dimensions[3] < 0)
  #declare half_dimensions[3] = 0;
#end
//
#declare part_dimensions[0]   = half_dimensions;
#declare part_dimensions[1]   = fraction_dimensions;
#declare right_object         = object{CL_rlp_make_concatenate(half_object,fraction_object,CL_medium_space,part_dimensions,CL_dimensions_out)} 
#declare right_dimensions     = CL_dimensions_out;
//
#declare part_dimensions[0]   = deriv_dimensions;
#declare part_dimensions[1]   = surd_dimensions;    
#declare left_object          = object{CL_rlp_make_concatenate(deriv_object,surd_object,CL_medium_space,part_dimensions,CL_dimensions_out)} 
//
#declare part_dimensions[0]   = CL_dimensions_out;
#declare part_dimensions[1]   = right_dimensions;
// Just testing passing an object for the middle part. Isn't necessary for this example since we could just pass " = "
#declare middle_part          = object{CL_rlp_make_text_string("# = ",CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                                                CL_dimensions_out)} 
#declare part_dimensions[2]   = CL_dimensions_out; 
//
#declare equation_object      = object{CL_rlp_make_equation(CL_tiny_space,CL_letter_adjust,CL_z1,CL_z2,CL_d_rad,CL_offs,CL_cham,CL_font_type,
                                       left_object,right_object,middle_part,"",CL_medium_space,part_dimensions,CL_dimensions_out)} 
//
// Top equation
// The make equation #macro produces an object, but instead of using CL_dimensions_out[2] to mean the superscript height, it denotes the
// x co-ordinate of the centre of the equals sign (or other relational string) 
//
object{equation_object translate <-CL_dimensions_out[2],5,0> pigment{rgb 1} finish{ambient 0 specular 1}}      
//
// Cylinder to show the equals sign is centred on zero.
//
cylinder{<0,-10,0> <0,10,0> 0.05 pigment{Green} }
//
  ////
 //  end example  //
//_______________//
light_source{<0,20,-25>, 4 fade_power 2 fade_distance 10
  #if(0)
    area_light x*2,y*2,8,8
    jitter adaptive 2
    area_illumination
  #end
            }

camera{orthographic location<-2.4,0,-100> look_at <-2.4,0,0> angle 15}

